@@ -115,7 +115,7 @@ def _del_jos_xprofiles(db, uidNumber):
 	db.query_rowcount(sql, data)
 
 
-def _update_jos_profiles_pw(db, userid, pwHash):
+def _update_jos_xprofiles_pw(db, userid, pwHash):
 	sql = "UPDATE `jos_xprofiles` SET `userPassword`=%s WHERE `uidNumber`=%s"
 	data = (pwHash, userid)
 	db.query_rowcount(sql, data)
@@ -126,6 +126,15 @@ def _update_jos_users_pw(db, userid, pwHash):
 	data = (pwHash, userid)
 	db.query_rowcount(sql, data)
 	
+
+def _update_jos_users_password(db, userid, pwHash):
+	sql = "UPDATE `jos_users_password` SET `passhash` = %s WHERE `user_id`=%s"
+	data = (pwHash, userid)
+	db.query_rowcount(sql, data)
+	
+
+#TODO jos_password_history??
+	
 	
 
 def _update_ldap_user_pw(username, uidNumber, pw):
@@ -142,14 +151,10 @@ def _update_ldap_user_pw(username, uidNumber, pw):
 
 	dn = "uid=" + username + ",ou=users," + hubLDAPBaseDN
 	
-	print "dn=" + dn
-
 	if pw != '*':
 		attrs = [ (ldap.MOD_REPLACE, 'userPassword', hubzero.config.passwords.MD5PasswordHash(pw)) ]
 	else:
 		attrs = [ (ldap.MOD_REPLACE, 'userPassword', "{MD5}*") ]
-	
-	print "update pw for ldap user dn: " + dn
 
 	l.modify_s(dn, attrs)	
 	
@@ -186,6 +191,7 @@ def _insert_jos_core_acl_groups_aro_map(db, jgidNumber, aroID):
 	data = ( jgidNumber, '', aroID)
 	db.query_rowcount(sql, data)
 
+
 def _insert_jos_xprofiles(db, hubname, uid, fname, mname, lname, username, email, pw, joomlagid, joomlagidNumber, gid, gidNumber, loginShell, homeDir):
 	
 	sql = "INSERT INTO `jos_xprofiles` (`uidNumber`, `name`, `username`, `email`, `registerDate`, `gidNumber`, `homeDirectory`, `loginShell`, "
@@ -230,7 +236,7 @@ def add_ldap_user(username, gecos, password, uidNumber, homeDir, loginShell, gid
 	elif '{MD5}' in password:
 		attrs['userPassword'] = password
 	else:
-		attrs['userPassword'] = '{MD5}*' # can user generate a value that will MD5 hash to '*'?
+		attrs['userPassword'] = '{MD5}*' # can user generate a value that will MD5 hash to, we think not '*'?
 	
 	attrs['uid'] = username
 	attrs['uidNumber'] = str(uidNumber)
@@ -262,8 +268,27 @@ def delete_ldap_user(username):
 
 
 def addhubuser(hubname, username, gecos, email, pw, pwdisabled, joomlagid, 
-               joomlagidNumber, gid, gidNumber, joomlauserid, loginShell, nocreatehome, homeDir, createGroup, 
-               skipldap):
+               joomlagidNumber, gid, gidNumber, uid, loginShell, nocreatehome, homeDir, createGroup, 
+               skipldap, skipcms):
+	"""
+	hubname - for machines that have more than one hub, otherwise this will use the default hub for a server
+	username - username to add
+	gecos - comma delimited info list on user, we curretnly only use first entry, which is a full name (fname, mname, lname)(
+	email - user email
+	pw - unencrypted password, we'll hash it as necessary using MD5
+	pwdisabled - should the PW be disabled?
+	joomlagid - specify hardcoded uidNumber for the user
+	joomlagidNumber - joomla group
+	gid - linux gid membership
+	gidNumber - linux gidNumber, corresponds to the group with the previous gid
+	uid - specify the jos_users PK ID, also is the uidNumber for the ldap (same thing)
+	loginShell - user's login shell
+	nocreatehome - skip creation of home directory
+	homeDir - path to user's home directory
+	createGroup - create a default group for new user, group name matches the username
+	skipldap - skip creating ldap info for this user
+	skipcms	- skip ldap info for this user
+	"""
 
 	# grab config options
 	dbPW = hubzero.config.webconfig.getWebConfigDBPassword()
@@ -272,14 +297,9 @@ def addhubuser(hubname, username, gecos, email, pw, pwdisabled, joomlagid,
 	
 	db =  hubzero.data.db.MySQLConnection("localhost", dbName, dbUserName, dbPW)
 
-	if _userExists(db, username):
-		print "User already exists: " + username
-		return (-1)
-
-
-	# parse gecos, it's a massive mess, but it's a comma delimited field used in the /etc/passwd file to store personal info
-	# it's comma delimited, and name is the first item in the list... right now we don't care about what else gets passed,
-	# but the first entry
+	# Parse gecos, it's a massive mess, but it's a holdover from the wild west days of unix and it's a comma delimited field 
+	# used in the /etc/passwd file to store personal info. It's comma delimited, and name is the first item in the list... 
+	# right now we don't care about what else gets passed, just the first entry
 	gecosFields = gecos.split(",")
 	fullName = gecosFields[0]
 	fullNameList = gecosFields[0].split(" ")
@@ -300,58 +320,72 @@ def addhubuser(hubname, username, gecos, email, pw, pwdisabled, joomlagid,
 		firstName = ""
 		middleName = ""
 		lastName = ""
-	
 
 	# usually homeDir will be blank, but not always
 	if homeDir == "":
 		homeDir = '/home/' + hubname + "/" + username
 
-
 	if pwdisabled: 
 		pw = "*"
 
-	# create linux group for user named same as username (need group insert here for xprofiles addition below)
-	if createGroup:
-		gidNumber = hubzero.utilities.group._insert_jos_xgroups(db, username, username + " group")	
-		gid = username # group name same as user
-
-
-	userID = _insert_jos_users(db, joomlauserid, fullName, username, email, pw, joomlagid, joomlagidNumber)
-	aroID = _insert_jos_core_acl_aro(db, userID, fullName)
-	_insert_jos_core_acl_groups_aro_map(db, joomlagidNumber, aroID)
-	_insert_jos_xprofiles(db, hubname, userID, firstName, middleName, lastName, username, email, pw, joomlagid, joomlagidNumber, gid, gidNumber, loginShell, homeDir)
-	_insert_jos_users_password(db, userID, pw)
-
-
-	# create linux group for user named same as username (need xgroups insert here because we need userid)
-	if createGroup:
-
-		hubzero.utilities.group._insert_jos_xgroups_members(db, gidNumber, userID)
-		hubzero.utilities.group._add_ldap_group(username, username + " group", gidNumber)
+	# check for unique username
+	if not skipcms:
+		# when inserting a cms user, with or without a ldap user, test the mysql database for unique usernames
+		if _userExists(db, username):
+			print "User '" + username + "' already exists in CMS"
+			return (1)
+	elif not skipldap:
+		# no cms and ldap, check ldap
+		if uidExists(username):
+			print "User '" + username + "' already exists in ldap"
+			return (2)
 	else:
+		print "skip cms insert and ldap, why bother?"
+		return(3)
+
+	if not createGroup:
 		gidNumber = 100 # default to the users group
+		gid = 'users'
+
+	if not skipcms:
 		
+		# create group for user named same as username (need group insert here for xprofiles addition below)
+		# Guess if user specifies a specific already created group for the new user and says create a new group for the 
+		# user, we'll override and create the new group with this logic
+		if createGroup:
+			gidNumber = hubzero.utilities.group._insert_jos_xgroups(db, username, username + " group")	
+			gid = username # group name same as user
 	
+		userID = _insert_jos_users(db, uid, fullName, username, email, pw, joomlagid, joomlagidNumber)
+		aroID = _insert_jos_core_acl_aro(db, userID, fullName)
+		_insert_jos_core_acl_groups_aro_map(db, joomlagidNumber, aroID)
+		_insert_jos_xprofiles(db, hubname, userID, firstName, middleName, lastName, username, email, pw, joomlagid, joomlagidNumber, gid, gidNumber, loginShell, homeDir)
+		_insert_jos_users_password(db, userID, pw)
+	
+		# add user to new group (need xgroups insert after user creation because we need userid, I'd prefer to keep all group logic in one place)
+		if createGroup:
+			hubzero.utilities.group._insert_jos_xgroups_members(db, gidNumber, userID)
+		
+		# create home directory
+		if not nocreatehome:
+			rc, procStdOut, procStdErr = hubzero.utilities.misc.exShellCommand(["mkdir", homeDir, "--mode", "0750"])
+			print procStdOut
+			if rc:
+				print procStdErr
+			
+			print "creating homedir: " + homeDir
+			
+			# set ownership
+			os.chown(homeDir, userID, gidNumber)
+			
+			# change permissions user full, noone else has anything
+			rc, procStdOut, procStdErr = hubzero.utilities.misc.exShellCommand(["chmod", '700', homeDir])
+			if rc:
+				print procStdErr
+
 	if not skipldap:
+		hubzero.utilities.group.add_ldap_group(username, username + " group", gidNumber)
 		add_ldap_user(username, gecos, pw, userID, homeDir, loginShell, gid, gidNumber)
-
-
-	# create home directory
-	if not nocreatehome:
-		rc, procStdOut, procStdErr = hubzero.utilities.misc.exShellCommand(["mkdir", homeDir, "--mode", "0750"])
-		print procStdOut
-		if rc:
-			print procStdErr
-		
-		print "creating homedir: " + homeDir
-		
-		# set ownership
-		os.chown(homeDir, userID, gidNumber)
-		
-		# change permissions user full, noone else has anything
-		rc, procStdOut, procStdErr = hubzero.utilities.misc.exShellCommand(["chmod", '700', homeDir])
-		if rc:
-			print procStdErr
 		
 	return(0)
 
@@ -376,7 +410,7 @@ def delhubuser(username):
 	delete_ldap_user(username)
 
 
-def updateUserPW(username, pw):
+def updateUserPW(username, pw, updateCMS = True, updateLDAP = True):
 	"""
 	pass plaintext pw, we'll hash it
 	"""
@@ -389,8 +423,74 @@ def updateUserPW(username, pw):
 	userid = _getUserID(db, username)
 	pwHash = hubzero.config.passwords.MD5PasswordHash(pw)
 	
-	_update_jos_profiles_pw(db, userid, pwHash)
-	_update_jos_users_pw(db, userid, pwHash)
-	_update_ldap_user_pw(username, userid, pw)
+	if updateCMS:
+		_update_jos_xprofiles_pw(db, userid, pwHash)
+		_update_jos_users_pw(db, userid, pwHash)
+		_update_jos_users_password(db, userid, pwHash)
 	
-	
\ No newline at end of file
+	if updateLDAP:
+		_update_ldap_user_pw(username, userid, pw)
+	
+
+def uidNumberExists(uidNumber):
+	"""
+	See if specified uidNumber is in the ldap user ou
+	"""
+
+	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn")
+	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
+	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn")	
+
+	l = ldap.open("localhost")
+	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
+
+	results = l.search_s('ou=users,' + hubLDAPBaseDN, ldap.SCOPE_SUBTREE, '(uidNumber=' + uidNumber + ')', None)
+
+	if not results:
+		return False
+	else:
+		return True
+
+
+def uidExists(uid):
+	"""
+	See if specified uid is in the ldap user ou
+	"""
+
+	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn")
+	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
+	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn")	
+
+	l = ldap.open("localhost")
+	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
+
+	results = l.search_s('ou=users,' + hubLDAPBaseDN, ldap.SCOPE_SUBTREE, '(uid=' + uid + ')', None)
+
+	if not results:
+		return False
+	else:
+		return True
+	
+	
+def getLDAPMaxuid():
+	"""
+	get max(uid) of all objects of class posixAccount in ou=Users
+	"""
+
+	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn")
+	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
+	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn")	
+
+	l = ldap.open("localhost")
+	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
+	
+	print "hubLDAPBaseDN=" + hubLDAPBaseDN
+	
+	results = l.search_s('ou=users,' + hubLDAPBaseDN, ldap.SCOPE_SUBTREE, '(objectclass=posixAccount)', ['uidNumber'])
+	
+	maxid = 0
+	for result in results:
+		x = int(result[1]['uidNumber'][0])
+		if x > maxid: maxid = x
+		
+	return maxid	
\ No newline at end of file
